﻿using System;
using System.ComponentModel;
using System.Windows.Threading;

namespace SciChartMvvm
{
    public interface IDataProvider
    {
        void SubscribeUpdates(Action<XyValues> onDataUpdated);
        void UnsubscribeUpdates();
    }

    public class DataProvider : IDataProvider
    {
        private readonly Random _random = new Random();
        private double _last;
        private readonly double _bias = 0.01;
        private int _i = 0;
        private DispatcherTimer _timer;

        public void SubscribeUpdates(Action<XyValues> onDataUpdated)
        {
            var designMode = (LicenseManager.UsageMode == LicenseUsageMode.Designtime);
            if (designMode)
            {
                return;
            }

            _timer = new DispatcherTimer(DispatcherPriority.Render) { Interval = TimeSpan.FromMilliseconds(20) };
            _timer.Tick += (s, e) =>
            {
                var xyValues = GenerateRandomWalk(10);
                onDataUpdated(xyValues);
            };
            _timer.Start();
        }

        public void UnsubscribeUpdates()
        {
            if (_timer != null && _timer.IsEnabled)
            {
                _timer.Stop();
            }
        }

        private XyValues GenerateRandomWalk(int count)
        {
            var values = new XyValues();
            for (var i = 0; i < count; i++)
            {
                var next = _last + (_random.NextDouble() - 0.5 + _bias);
                _last = next;
                values.XValues.Add(_i++);
                values.DateTimeValues.Add(DateTime.Now);
                values.YValues.Add(next);
            }

            return values;
        }
    }
}