﻿using System;
using System.Collections.ObjectModel;
using System.Windows.Media;
using Microsoft.Practices.Prism.Commands;
using SciChart.Charting.Model.ChartSeries;
using SciChart.Charting.Model.DataSeries;
using SciChart.Charting.Visuals.Axes;
using SciChart.Data.Model;

namespace SciChartMvvm
{
    public class MainViewModel : BindableObject
    {
        private string _chartTitle;
        private bool _enableZoom = true;
        private bool _enablePan;
        private IAxisViewModel _xAxis;
        private IAxisViewModel _yAxis;
        private IDataSeries<DateTime, double> _dataSeries;
        private ObservableCollection<IRenderableSeriesViewModel> _renderableSeries;
        private IDataProvider _dataProvider;

        public string ChartTitle
        {
            get { return _chartTitle; }
            set
            {
                if (_chartTitle != value)
                {
                    _chartTitle = value;
                    OnPropertyChanged("ChartTitle");
                }
            }
        }

        public bool EnableZoom
        {
            get { return _enableZoom; }
            set
            {
                if (_enableZoom != value)
                {
                    _enableZoom = value;
                    if (_enableZoom)
                    {
                        EnablePan = false;
                    }

                    OnPropertyChanged("EnableZoom");
                }
            }
        }


        public bool EnablePan
        {
            get { return _enablePan; }
            set
            {
                if (_enablePan != value)
                {
                    _enablePan = value;
                    if (_enablePan)
                    {
                        EnableZoom = false;
                    }

                    OnPropertyChanged("EnablePan");
                }
            }
        }

        public IAxisViewModel XAxis
        {
            get { return _xAxis; }
            set
            {
                if (_xAxis != value)
                {
                    _xAxis = value;
                    OnPropertyChanged("XAxis");
                }
            }
        }

        public IAxisViewModel YAxis
        {
            get { return _yAxis; }
            set
            {
                if (_yAxis != value)
                {
                    _yAxis = value;
                    OnPropertyChanged("YAxis");
                }
            }
        }

        public ObservableCollection<IRenderableSeriesViewModel> RenderableSeries
        {
            get { return _renderableSeries; }
            set
            {
                if (_renderableSeries != value)
                {
                    _renderableSeries = value;
                    OnPropertyChanged("RenderableSeries");
                }
            }
        }

        public DelegateCommand StartCommand { get; }
        public DelegateCommand StopCommand { get; }

        private void CreateAxes()
        {
            XAxis = new DateTimeAxisViewModel
            {
                AxisTitle = "X-Achse",
            };
            YAxis = new NumericAxisViewModel
            {
                AxisTitle = "Y-Achse",
                AxisAlignment = AxisAlignment.Left
            };
        }

        private void CreateRenderableSeries()
        {
            _dataSeries = new XyDataSeries<DateTime, double>
            {
                SeriesName = "TestingSeries"
            };
            RenderableSeries = new ObservableCollection<IRenderableSeriesViewModel>
            {
                new LineRenderableSeriesViewModel
                {
                    StrokeThickness = 2,
                    Stroke = Colors.SteelBlue,
                    DataSeries = _dataSeries,
                    AntiAliasing = false
                }
            };
        }

        private void OnStart()
        {
            if (_dataProvider == null)
            {
                _dataProvider = new DataProvider();
            }

            _dataProvider.SubscribeUpdates(newValues =>
            {
                _dataSeries.Append(newValues.DateTimeValues, newValues.YValues);
                _dataSeries.InvalidateParentSurface(RangeMode.ZoomToFit);
            });
        }

        private void OnStop()
        {
            _dataProvider.UnsubscribeUpdates();
        }

        public MainViewModel()
        {
            ChartTitle = "Hello SciChart!";

            StartCommand = new DelegateCommand(OnStart);
            StopCommand = new DelegateCommand(OnStop);

            CreateAxes();
            CreateRenderableSeries();
        }
    }
}